{% extends "base.html" %}

{% block title %}
	&raquo; Non-Core Contracts &raquo; {{ contract.name }} &raquo; Add Rate Script
{% endblock %}

{% block nav %}
	<a href="/non_core_contracts">Non-Core Contracts</a> &raquo; 
	<a href="/non_core_contracts/{{contract.id}}">{{contract.name}}</a>
	&raquo; Add Rate Scripts
{% endblock %}

{% block content %}
	<form method="post">
		<fieldset>
			<legend>Add a rate script</legend>	
			<input type="hidden" name="non_core_contract_id" value="{{contract.id}}">
			
			<fieldset>
				<legend>Start Date</legend>
				{{ input_date('start', initial_date) }}
			</fieldset>
			
			<input type="submit" value="Add">
		</fieldset>
	</form>
{% endblock %}
