{% extends "base.html" %}

{% block title %}
	&raquo; User Roles
{% endblock %}

{% block nav %}
	User Roles
{% endblock %}

{% block content %}
	<table>
		<thead>
			<th>Code</th>
		</thead>
		<tbody>
			{% for user_role in user_roles %}
				<tr>
					<td>
						<a href="/user_roles/{{user_role.id}}">{{user_role.code}}</a>
					</td>
				</tr>
			{% endfor %}
		</tbody>
	</table>
{% endblock %}
