{% extends "base.html" %}

{% block title %}
	&raquo; MOP Contracts &raquo; {{rate_script.contract.name}} &raquo; 
	{{rate_script.start_date|hh_format}} &raquo; Edit
{% endblock %}

{% block nav %}
	<a href="/e/mop_contracts">MOP Contracts</a> &raquo; 
	<a
			href="/e/mop_contracts/{{rate_script.contract.id}}"
	>{{rate_script.contract.name}}</a> &raquo; 
	<a
		 href="/e/mop_rate_scripts/{{rate_script.id}}"
	>{{rate_script.start_date|hh_format }}</a> &raquo; Edit
{% endblock %}

{% block content %}
	{% if request.method == 'GET' and request.values.delete %}
		
		<form method="post" action="/e/mop_rate_scripts/{{rate_script.id}}/edit">
			<fieldset>
				<legend>Are you sure you want to delete this rate script?</legend>
				<input type="submit" name="delete" value="Delete">
				<a href="/e/mop_rate_scripts/{{rate_script.id}}/edit">Cancel</a>
			</fieldset>
		</form>
	
	{% else %}
	
		<form method="post" action="/e/mop_rate_scripts/{{rate_script.id}}/edit">
			<fieldset>
				<legend>Update Rate Script</legend>	
				<fieldset>
					<legend>Start date</legend>
					{{input_date('start', rate_script.start_date)}}
				</fieldset>
				<fieldset>
					<legend>Finish date</legend>
					<label>
						Ended?
						{{ input_checkbox( 'has_finished', rate_script.finish_date != None)}}
					</label>
					{{input_date('finish', rate_script.finish_date)}}
				</fieldset>
				<fieldset>
					<legend>Script</legend>
					{{input_textarea('script', rate_script.script, 40, 80, show_pos=True)}}
				</fieldset>
				<input type="submit" value="Update">
			</fieldset>
		</form>

		<h4>Example</h4>

		<pre>{{rate_script_example}}</pre>
	
		<form>
			<fieldset>
				<legend>Delete this Rate Script</legend>	
				<input type="submit" name="delete" value="Delete">
			</fieldset>
		</form>

	{% endif %}
{% endblock %}
